import sys

class Reporter(object):
    def __init__(self):
        self.failed_scenarios = []
        self.scenarios_and_its_fails = {}

    def wrt(self, what):
        if isinstance(what, unicode):
            what = what.encode('utf-8')
        sys.stdout.write(what)

    def store_failed_step(self, step):
        if step.failed and step.scenario not in self.failed_scenarios:
            self.scenarios_and_its_fails[step.scenario] = step.why
            self.failed_scenarios.append(step.scenario)

    def print_scenario_running(self, scenario):
        pass

    def print_scenario_ran(self, scenario):
        pass

    def print_end(self, total):
        if total.scenarios_passed < total.scenarios_ran:
            self.wrt("\n")
            self.wrt("\n")
            for scenario in self.failed_scenarios:
                reason = self.scenarios_and_its_fails[scenario]
                self.wrt(str(reason.step))
                self.wrt("\n")
                self.wrt(reason.traceback)

        self.wrt("\n")
        word = total.features_ran > 1 and "features" or "feature"
        self.wrt("%d %s (%d passed)\n" % (
            total.features_ran,
            word,
            total.features_passed))

        word = total.scenarios_ran > 1 and "scenarios" or "scenario"
        self.wrt("%d %s (%d passed)\n" % (
            total.scenarios_ran,
            word,
            total.scenarios_passed))

        steps_details = []
        for kind in "failed", "skipped", "undefined":
            attr = 'steps_%s' % kind
            stotal = getattr(total, attr)
            if stotal:
                steps_details.append("%d %s" % (stotal, kind))

        steps_details.append("%d passed" % total.steps_passed)
        word = total.steps > 1 and "steps" or "step"
        self.wrt("%d %s (%s)\n" % (
            total.steps,
            word,
            ", ".join(steps_details)))



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERsteps.pyCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timedelta
from lettuce.terrain import after
from lettuce.terrain import before
from xml.dom import minidom
from lettuce.strings import utf8_string


def wrt_output(filename, content):
    f = open(filename, "w")
    if isinstance(content, unicode):
        content = content.encode('utf-8')

    f.write(content)
    f.close()


def total_seconds(td):
    return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6) / 1e6


def enable(filename=None):

    doc = minidom.Document()
    root = doc.createElement("testsuite")
    root.setAttribute("name", "lettuce")
    root.setAttribute("hostname", "localhost")
    root.setAttribute("timestamp", datetime.now().strftime("%Y-%m-%dT%H:%M:%S"))
    output_filename = filename or "lettucetests.xml"

    @before.each_step
    def time_step(step):
        step.started = datetime.now()

    @after.each_step
    def create_test_case_step(step):
        if step.scenario.outlines:
            return

        classname = utf8_string(u"%s : %s" % (step.scenario.feature.name, step.scenario.name))
        tc = doc.createElement("testcase")
        tc.setAttribute("classname", classname)
        tc.setAttribute("name", step.sentence.encode('utf-8'))
        try:
            tc.setAttribute("time", str(total_seconds((datetime.now() - step.started))))
        except AttributeError:
            tc.setAttribute("time", str(total_seconds(timedelta(seconds=0))))

        if not step.ran:
            skip = doc.createElement("skipped")
            skip.setAttribute("type", "UndefinedStep(%s)" % step.sentence)
            tc.appendChild(skip)

        if step.failed:
            cdata = doc.createCDATASection(step.why.traceback.encode('utf-8'))
            failure = doc.createElement("failure")
            if hasattr(step.why, 'cause'):
                failure.setAttribute("message", step.why.cause.encode('utf-8'))
            failure.setAttribute("type", step.why.exception.__class__.__name__.encode('utf-8'))
            failure.appendChild(cdata)
            tc.appendChild(failure)

        root.appendChild(tc)

    @before.outline
    def time_outline(scenario, order, outline, reasons_to_fail):
        scenario.outline_started = datetime.now()
        pass

    @after.outline
    def create_test_case_outline(scenario, order, outline, reasons_to_fail):
        classname = "%s : %s" % (scenario.feature.name, scenario.name)
        tc = doc.createElement("testcase")
        tc.setAttribute("classname", classname)
        tc.setAttribute("name", u'| %s |' % u' | '.join(outline.values()))
        tc.setAttribute("time", str(total_seconds((datetime.now() - scenario.outline_started))))

        for reason_to_fail in reasons_to_fail:
            cdata = doc.createCDATASection(reason_to_fail.traceback)
            failure = doc.createElement("failure")
            failure.setAttribute("message", reason_to_fail.cause)
            failure.appendChild(cdata)
            tc.appendChild(failure)

        root.appendChild(tc)

    @after.all
    def output_xml(total):
        root.setAttribute("tests", str(total.steps))
        root.setAttribute("failures", str(total.steps_failed))
        root.setAttribute("errors", '0')
        root.setAttribute("time", '0')
        doc.appendChild(root)
        wrt_output(output_filename, doc.toxml())



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import traceback
from lettuce.strings import utf8_string


class NoDefinitionFound(Exception):
    """ Exception raised by lettuce.core.Step, when trying to solve a
    Step, but does not find a suitable step definition.

    This exception should never blow on user's face. It used merely yo
    lettuce can filter undefined steps.
    """
    def __init__(self, step):
        self.step = step
        super(NoDefinitionFound, self).__init__(
            'The step r"%s" is not defined' % self.step.sentence)


class ReasonToFail(object):
    """ Exception that contains detailed information about a
    AssertionError raised within a step definition.  With these data
    lettuce show detailed traceback to user in a nice representation.
    """
    def __init__(self, step, exc):
        self.step = step
        self.exception = exc
        if isinstance(exc.message, basestring):
            self.cause = utf8_string(exc.message)
        self.traceback = utf8_string(traceback.format_exc(exc))


class LettuceSyntaxError(SyntaxError):
    def __init__(self, filename, string):
        self.filename = filename
        self.msg = "Syntax error at: %s\n%s\n" % (filename, string)


class StepLoadingError(Exception):
    """Raised when a step cannot be loaded."""
    pass



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


LANGUAGES = {
    'en': {
        'examples': u'Examples|Scenarios',
        'feature': u'Feature',
        'name': u'English',
        'native': u'English',
        'scenario': u'Scenario',
        'scenario_outline': u'Scenario Outline',
        'scenario_separator': u'(Scenario Outline|Scenario)',
        'background': u'(?:Background)',
    },
    'pt-br': {
        'examples': u'Exemplos|Cenários',
        'feature': u'Funcionalidade',
        'name': u'Portuguese',
        'native': u'Português',
        'scenario': u'Cenário|Cenario',
        'scenario_outline': u'Esquema do Cenário|Esquema do Cenario',
        'scenario_separator': u'(Esquema do Cenário|Esquema do Cenario|Cenario|Cenário)',
        'background': u'(?:Contexto|Considerações)',
    },
    'pl': {
        'examples': u'Przykład',
        'feature': u'Właściwość',
        'name': u'Polish',
        'native': u'Polski',
        'scenario': u'Scenariusz',
        'scenario_outline': u'Zarys Scenariusza',
        'scenario_separator': u'(Zarys Scenariusza|Scenariusz)',
        'background': u'(?:Background)',
    },
    'ca': {
        'examples': u'Exemples',
        'feature': u'Funcionalitat',
        'name': u'Catalan',
        'native': u'Català',
        'scenario': u'Escenari',
        'scenario_outline': u"Esquema d'Escenari",
        'scenario_separator': u"(Esquema d'Escenari|Escenari)",
        'background': u'(?:Background)',
    },
    'es': {
        'examples': u'Ejemplos',
        'feature': u'Funcionalidad',
        'name': u'Spanish',
        'native': u'Español',
        'scenario': u'Escenario',
        'scenario_outline': u'Esquema de Escenario',
        'scenario_separator': u'(Esquema de Escenario|Escenario)',
        'background': u'(?:Contexto|Consideraciones)',
    },
    'hu': {
        'examples': u'Példák',
        'feature': u'Jellemző',
        'name': u'Hungarian',
        'native': u'Magyar',
        'scenario': u'Forgatókönyv',
        'scenario_outline': u'Forgatókönyv vázlat',
        'scenario_separator': u'(Forgatókönyv|Forgatókönyv vázlat)',
        'background': u'(?:Background)',
    },
    'fr': {
        'examples': u'Exemples|Scénarios',
        'feature': u'Fonctionnalité|Fonction',
        'name': u'French',
        'native': u'Français',
        'scenario': u'Scénario',
        'scenario_outline': u'Plan de Scénario|Plan du Scénario',
        'scenario_separator': u'(Plan de Scénario|Plan du Scénario|Scénario)',
        'background': u'(?:Background)',
    },
    'de': {
        'examples': u'Beispiele|Szenarios',
        'feature': u'Funktionalität|Funktion',
        'name': u'German',
        'native': u'Deutsch',
        'scenario': u'Szenario',
        'scenario_outline': u'Szenario-Zusammenfassung|Zusammenfassung',
        'scenario_separator': u'(Szenario-Zusammenfassung|Zusammenfassung)',
        'background': u'(?:Background)',
    },
    'ja': {
        'examples': u'例',
        'feature': u'フィーチャ',
        'name': u'Japanese',
        'native': u'日本語',
        'scenario': u'シナリオ',
        'scenario_outline': u'シナリオアウトライン|シナリオテンプレート|テンプレ|シナリオテンプレ',
        'scenario_separator': u'(シナリオ|シナリオアウトライン|シナリオテンプレート|テンプレ|シナリオテンプレ)',
        'background': u'(?:Background)',
    },
    'tr': {
        'examples': u'Örnekler',
        'feature': u'Özellik',
        'name': u'Turkish',
        'native': u'Türkçe',
        'scenario': u'Senaryo',
        'scenario_outline': u'Senaryo taslağı|Senaryo Taslağı',
        'scenario_separator': u'(Senaryo taslağı|Senaryo Taslağı|Senaryo)',
        'background': u'(?:Background)',
    },
    'zh-CN': {
        'examples': u'例如|场景集',
        'feature': u'特性',
        'name': u'Simplified Chinese',
        'native': u'简体中文',
        'scenario': u'场景',
        'scenario_outline': u'场景模板',
        'scenario_separator': u'(场景模板|场景)',
        'background': u'(?:背景)',
    },
    'zh-TW': {
        'examples': u'例如|場景集',
        'feature': u'特性',
        'name': u'Traditional Chinese',
        'native': u'繁體中文',
        'scenario': u'場景',
        'scenario_outline': u'場景模板',
        'scenario_separator': u'(場景模板|場景)',
        'background': u'(?:背景)',
    },
    'ru': {
        'examples': u'Примеры|Сценарии',
        'feature': u'Функционал',
        'name': u'Russian',
        'native': u'Русский',
        'scenario': u'Сценарий',
        'scenario_outline': u'Структура сценария',
        'scenario_separator': u'(Структура сценария|Сценарий)',
        'background': u'(?:Background)',
    },
    'uk': {
        'examples': u'Приклади|Сценарії',
        'feature': u'Функціонал',
        'name': u'Ukrainian',
        'native': u'Українська',
        'scenario': u'Сценарій',
        'scenario_outline': u'Структура сценарію',
        'scenario_separator': u'(Структура сценарію|Сценарій)',
        'background': u'(?:Background)',
    },
    'it': {
        'examples': u'Esempi|Scenari|Scenarii',
        'feature': u'Funzionalità|Funzione',
        'name': u'Italian',
        'native': u'Italiano',
        'scenario': u'Scenario',
        'scenario_outline': u'Schema di Scenario|Piano di Scenario',
        'scenario_separator': u'(Schema di Scenario|Piano di Scenario|Scenario)',
        'background': u'(?:Background)',
    },
    'no': {
        'examples': u'Eksempler',
        'feature': u'Egenskaper',
        'name': u'Norwegian',
        'native': u'Norsk',
        'scenario': u'Situasjon',
        'scenario_outline': u'Situasjon Oversikt',
        'scenario_separator': u'(Situasjon Oversikt|Situasjon)',
        'background': u'(?:Bakgrunn)',
    },
    'sv': {
        'examples': u'Exempel|Scenarion',
        'feature': u'Egenskaper',
        'name': u'Swedish',
        'native': u'Svenska',
        'scenario': u'Scenario',
        'scenario_outline': u'Scenarioöversikt',
        'scenario_separator': u'(Scenarioöversikt|Scenario)',
        'background': u'(?:Context)',
    },
}



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import time
import unicodedata


def utf8_string(s):
    if isinstance(s, str):
        s = s.decode("utf-8")

    elif isinstance(s, unicode):
        s = s.encode("utf-8")
    return s


def escape_if_necessary(what):
    what = unicode(what)
    if len(what) is 1:
        what = u"[%s]" % what

    return what


def get_stripped_lines(string, ignore_lines_starting_with=''):
    string = unicode(string)
    lines = [unicode(l.strip()) for l in string.splitlines()]
    if ignore_lines_starting_with:
        filter_func = lambda x: x and not x.startswith(
            ignore_lines_starting_with)
    else:
        filter_func = lambda x: x

    lines = filter(filter_func, lines)

    return lines


def split_wisely(string, sep, strip=False):
    string = unicode(string)
    if strip:
        string = string.strip()
    else:
        string = string.strip("\n")
    sep = unicode(sep)

    regex = re.compile(escape_if_necessary(sep),  re.UNICODE | re.M | re.I)

    items = filter(lambda x: x, regex.split(string))
    if strip:
        items = [i.strip() for i in items]
    else:
        items = [i.strip("\n") for i in items]

    return [unicode(i) for i in items]


def wise_startswith(string, seed):
    string = unicode(string).strip()
    seed = unicode(seed)
    regex = u"^%s" % re.escape(seed)
    return bool(re.search(regex, string, re.I))


def remove_it(string, what):
    return unicode(re.sub(unicode(what), "", unicode(string)).strip())


def column_width(string):
    l = 0
    for c in unicode(string):
        if unicodedata.east_asian_width(c) in "WF":
            l += 2
        else:
            l += 1
    return l


def rfill(string, times, char=u" ", append=u""):
    string = unicode(string)
    missing = times - column_width(string)
    for x in range(missing):
        string += char

    return unicode(string) + unicode(append)


def getlen(string):
    return column_width(unicode(string)) + 1


def dicts_to_string(dicts, order):
    escape = "#{%s}" % unicode(time.time())

    def enline(line):
        return unicode(line).replace("|", escape)

    def deline(line):
        return line.replace(escape, '\\|')

    keys_and_sizes = dict([(k, getlen(k)) for k in dicts[0].keys()])
    for key in keys_and_sizes:
        for data in dicts:
            current_size = keys_and_sizes[key]
            value = unicode(data.get(key, ''))
            size = getlen(value)
            if size > current_size:
                keys_and_sizes[key] = size

    names = []
    for key in order:
        size = keys_and_sizes[key]
        name = u" %s" % rfill(key, size)
        names.append(enline(name))

    table = [u"|%s|" % "|".join(names)]
    for data in dicts:
        names = []
        for key in order:
            value = data.get(key, '')
            size = keys_and_sizes[key]
            names.append(enline(u" %s" % rfill(value, size)))

        table.append(u"|%s|" % "|".join(names))

    return deline(u"\n".join(table) + u"\n")


def parse_hashes(lines):
    escape = "#{%s}" % unicode(time.time())

    def enline(line):
        return unicode(line.replace("\\|", escape)).strip()

    def deline(line):
        return line.replace(escape, '|')

    def discard_comments(lines):
        return [line for line in lines if not line.startswith('#')]

    lines = discard_comments(lines)
    lines = map(enline, lines)

    keys = []
    hashes = []
    if lines:
        first_line = lines.pop(0)
        keys = split_wisely(first_line, u"|", True)
        keys = map(deline, keys)

        for line in lines:
            values = split_wisely(line, u"|", True)
            values = map(deline, values)
            hashes.append(dict(zip(keys, values)))

    return keys, hashes


def parse_multiline(lines):
    multilines = []
    in_multiline = False
    for line in lines:
        if line == '"""':
            in_multiline = not in_multiline
        elif in_multiline:
            if line.startswith('"'):
                line = line[1:]
            if line.endswith('"'):
                line = line[:-1]
            multilines.append(line)
    return u'\n'.join(multilines)



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import threading
import traceback

world = threading.local()
world._set = False


def _function_matches(one, other):
    return (os.path.abspath(one.func_code.co_filename) == os.path.abspath(other.func_code.co_filename) and
            one.func_code.co_firstlineno == other.func_code.co_firstlineno)


class CallbackDict(dict):
    def append_to(self, where, when, function):
        if not any(_function_matches(o, function) for o in self[where][when]):
            self[where][when].append(function)

    def clear(self):
        for name, action_dict in self.items():
            for callback_list in action_dict.values():
                callback_list[:] = []


STEP_REGISTRY = {}
CALLBACK_REGISTRY = CallbackDict(
    {
        'all': {
            'before': [],
            'after': [],
        },
        'step': {
            'before_each': [],
            'after_each': [],
        },
        'scenario': {
            'before_each': [],
            'after_each': [],
            'outline': [],
        },
        'background': {
            'before_each': [],
            'after_each': [],
        },
        'feature': {
            'before_each': [],
            'after_each': [],
        },
        'app': {
            'before_each': [],
            'after_each': [],
        },
        'harvest': {
            'before': [],
            'after': [],
        },
        'handle_request': {
            'before': [],
            'after': [],
        },
        'runserver': {
            'before': [],
            'after': [],
        },
    },
)


def call_hook(situation, kind, *args, **kw):
    for callback in CALLBACK_REGISTRY[kind][situation]:
        try:
            callback(*args, **kw)
        except Exception, e:
            print "=" * 1000
            traceback.print_exc(e)
            print
            raise


def clear():
    STEP_REGISTRY.clear()
    CALLBACK_REGISTRY.clear()



from lettuce import world, step
from lettuce.django import mail
from lettuce.django import django_url

from nose.tools import assert_equals


@step(u'I visit "([^"]*)"')
def visit(step, url):
    world.browser.visit(django_url(url))


@step(u'I see "([^"]*)"')
def see(step, text):
    assert world.browser.is_text_present(text)


@step(u'an email is sent to "([^"]*?)" with subject "([^"]*)"')
def email_sent(step, to, subject):
    message = mail.queue.get(True, timeout=5)
    assert_equals(message.subject, subject)
    assert_equals(message.recipients(), [to])
